#!/usr/bin/env python

import numpy as np
from matplotlib import animation
from matplotlib import pyplot as plt
from matplotlib.backends.qt_compat import QtWidgets
from matplotlib.widgets import Slider

from logslider import LogSlider, LogSlider0

π = np.pi

S = 10.001

ω0 = 1.00
ω = 1.15
β = 0.01
x0 = 1.0
v0 = 0.0
F = 1.0


def oscillations(tt, ω, ω0, β, x0, v0):
    A1 = F * (ω0**2 - ω**2) / ((ω0**2 - ω**2)**2 + 4 * β**2 * ω**2)
    B1 = 2 * F * β * ω / ((ω0**2 - ω**2)**2 + 4 * β**2 * ω**2)
    x1 = A1 * np.cos(ω * tt) + B1 * np.sin(ω * tt)
    x0 -= A1
    v0 -= B1 * ω
    decay = np.exp(-β * tt)
    if β < ω0:
        ω = np.sqrt(ω0**2 - β**2)
        A2 = x0
        B2 = (v0 + β * x0) / ω
        x2 = (A2 * np.cos(ω * tt) + B2 * np.sin(ω * tt)) * decay
    elif β > ω0:
        ωi = np.sqrt(β**2 - ω0**2)
        A2 = x0
        B2 = (v0 + β * x0) / ωi
        x2 = (A2 * np.cosh(ωi * tt) + B2 * np.sinh(ωi * tt)) * decay
    else:
        A2 = x0
        B2 = v0 + β * x0
        x2 = (A2 + B2 * tt) * decay
    return x1, x2, x1 + x2


# plt.style.use('dark_background')

fig, ax = plt.subplots()

manager = fig.canvas.manager
manager.window.findChild(QtWidgets.QToolBar).setVisible(False)
manager.set_window_title("Drgania wymuszone")
manager.full_screen_toggle()

ax.axhline(0, color='0.3', lw=0.7)

a1 = ax.axhline(1e1, color='0.8', lw=0.3)
a2 = ax.axhline(-1e1, color='0.8', lw=0.3)

tt = np.linspace(0., 10**2.5, 10001)

x1, x2, xx = oscillations(tt, ω, ω0, β, x0, v0)
linex1, = ax.plot(tt, x1, lw=0.7, color='C1')
linex2, = ax.plot(tt, x2, lw=0.7, color='C3')
linexx, = ax.plot(tt, xx, lw=2.0, color='C0')

ax.set_ylim(-S, S)
ax.set_yscale('symlog', linthresh=1e1)

ax.set_xlabel('$t$ [s]')
ax.set_xlim(tt[0], tt[-1])

fig.tight_layout()

fig.subplots_adjust(bottom=0.24, top=0.98, left=0.07, right=0.96)
tax = fig.add_axes([0.17, 0.10, 0.75, 0.03])
ωax = fig.add_axes([0.17, 0.07, 0.75, 0.03])
βax = fig.add_axes([0.17, 0.04, 0.75, 0.03])
xax = fig.add_axes([0.17, 0.01, 0.32, 0.03])
vax = fig.add_axes([0.60, 0.01, 0.32, 0.03])

sax = fig.add_axes([0.97, 0.64, 0.02, 0.32])
lax = fig.add_axes([0.97, 0.24, 0.02, 0.32])


sst = {
    'track_color': '0.7',
    'color': '0.7',
    'initcolor': 'black',
}

time_slider = LogSlider(ax=tax, label='Czas całkowity [s]', valmin=0.5, valmax=3, valinit=2.5, valstep=0.125, **sst)
scale_slider = LogSlider(ax=sax, label='A', valmin=0, valmax=3, valinit=np.log10(S), valstep=0.02, **sst, orientation='vertical')
log_slider = Slider(ax=lax, label='log', valmin=-12, valmax=1, valinit=1, valstep=1, **sst, orientation='vertical')

omega_slider = Slider(ax=ωax, label='$\\omega$ [Hz]', valmin=0.0, valmax=2.0, valinit=ω, valstep=0.01, **sst)
beta_slider = LogSlider0(
    ax=βax, label='Tłumienie $\\beta/\\omega_0$', valmin=-3, valmax=1, valstep=0.01, valinit=np.log10(2**(-0.5)), **sst
)
x0_slider = Slider(ax=xax, label='$x_0$', valmin=0., valmax=2.0, valstep=0.1, valinit=x0, **sst)
v0_slider = Slider(ax=vax, label='$v_0$', valmin=-1.0, valmax=1.0, valstep=0.25, valinit=v0, **sst)

beta_slider.set_val(np.log10(β))


def on_slider(val):
    global tt
    tt = np.arange(0., time_slider.val + 0.005, 0.01)
    ax.set_xlim(tt[0], tt[-1])
    ω = omega_slider.val
    β = beta_slider.val
    x0 = x0_slider.val
    v0 = v0_slider.val
    x1, x2, xx = oscillations(tt, ω, ω0, β, x0, v0)
    linex1.set_data(tt, x1)
    linex2.set_data(tt, x2)
    linexx.set_data(tt, xx)
    lx = 10**log_slider.val
    a1.set_ydata(lx)
    a2.set_ydata(-lx)
    S = scale_slider.val
    ax.set_ylim(-S, S)
    ax.set_yscale('symlog', linthresh=lx)
    fig.canvas.draw_idle()


time_slider.on_changed(on_slider)
log_slider.on_changed(on_slider)
scale_slider.on_changed(on_slider)
omega_slider.on_changed(on_slider)
beta_slider.on_changed(on_slider)
x0_slider.on_changed(on_slider)
v0_slider.on_changed(on_slider)

plt.show()
